package tp4509.graphique;

import static dm4509.common.Log.LOGGER_NAME_COMM;
import static dm4509.common.Log.LOGGER_NAME_GEN;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.channels.SocketChannel;

import javax.swing.SwingUtilities;

import org.apache.log4j.Level;

import dm4509.common.Log;
import tp4509.picross.FileFormatException;
import tp4509.picross.PicrossApi;
import tp4509.tcp.ThreadClient;
/**
 * Classe d'application coté client.
 * @author Eric Lallet.
 *
 */
public final class AppliSwing {
	/**
	 * number of expected arguments of main method.
	 */
	private static final int NBARGS = 2;
	/**
	 * position of the hostname in argv[].
	 */
	private static  final int HOSTARG = 0;
	/**
	 * position of the port number in argv[].
	 */
	private static  final int PORTARG = 1;
	
	/**
	 * 
	 */
	private AppliSwing() {
		
	}
	
	/**
	 * Le programme principale de l'application coté client.
	 * @param argv
	 *     trois usage possibles:
	 *     1. aucun argument: lance une partie solitaire sur le fichier "./data/poisson.txt".
	 *     2. un seul argument: lance une partie solitaire sur le fichier argv[0].
	 *     3. deux arguments: lance une partie réseau su le serveur à l'adresse argv[0]:argv[1]
	 * 
	 * @throws NumberFormatException
	 *     Exception levée quand argv[1] devant contenir le numéro du port n'est pas un entier.
	 * @throws IOException
	 *     Exception levée pour les erreurs d'entrée/sortie sur le fichier de la grille.
	 * @throws FileFormatException
	 *     Exception levée quand le fichier de la grille est mal formaté.
	 */
	public static void main(final String[] argv) throws NumberFormatException, IOException, FileFormatException {
		PicrossApi picrossApi; 
		if (argv.length == 0) {
			picrossApi = new PicrossApi("./data/poisson.txt"); 
			SwingUtilities.invokeLater(new RunnableSwing(picrossApi));
			return;
		}
		if (argv.length == 1) {
			picrossApi = new PicrossApi(argv[0]); 
			SwingUtilities.invokeLater(new RunnableSwing(picrossApi));
			return;
		}
		
		if (argv.length != NBARGS) {
			System.err.println("Usage: AppliSwing <serverHostname> <serverPort>");
			return;
		}
		// picross en réseau.
		Log.configureALogger(LOGGER_NAME_COMM, Level.TRACE);
		Log.configureALogger(LOGGER_NAME_GEN, Level.TRACE);
		
		// Ouverture du canal client connecté au serveur de jeu. Il est utilisé par le thread de Swing
		// pour les envois de message, et pas le ThreadClient pour les réceptions.
		SocketChannel rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName(argv[HOSTARG]),  Integer.parseInt(argv[PORTARG])));
	
		picrossApi = new PicrossApi(rwChan); 
		RunnableSwing tacheSwing = new RunnableSwing(picrossApi);
		
		// lancement du thread SWING: gestion du joueur et envoi des messages vers le serveur.
		SwingUtilities.invokeLater(tacheSwing);
		
		// lancement du thread recevant et traitant les messages du serveur.
		// TODO Question 3.3.3 première partie


	}

}
