package tp4509.graphique;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Classe pour dessiner chaque case du jeu.
 * Une case peut contenir 4 éléments différents:
 *  1; une case blanche;
 *  2: une case noire;
 *  3. une case barrée;
 *  4. un texte.
 * @author Epsi
 *
 */
public class CaseGrille extends JPanel {
 
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	/**
	 * Taille de la fonte pour le X qui barre les cases.
	 */
	private static final int FONTZIZEX = 25;
	
	/**
	 * Taille de la fonte des textes affichés (y compris les indices).
	 */
	private static final int FONTZIZE = 12;
	
	/**
	 * Label contenant le texte de la case.
	 */
	private JLabel labelTexte;
	/**
	 * Panel contenant la zone interne à colorer.
	 */
	private JPanel couleur;
	

	/**
	 * Constructeur.
	 * La case est composé de 3 parties:
	 *  1. le panel global;
	 *  2. un sous panel qui va determiné le couleur de la case;
	 *  3. un labal encapsulé pour pouvoir ajouter une texte.
	 *  Le constrcuteur délivre une case blanche sans bordure avec un texte vide. 
	 */
	public CaseGrille() {

		// le panel global à la latille TAILLECEL x TAILLECEL 
		this.setPreferredSize(WindowSwing.DIMCEL1);
		this.setLayout(new BorderLayout());
		this.setBackground(Color.WHITE);
		
		// un sous panel pour la zone colorée
		couleur = new JPanel();
		couleur.setPreferredSize(WindowSwing.DIMCEL2);
		couleur.setBorder(BorderFactory.createLineBorder(Color.WHITE, 2)); // la zone colorée est entourée d'une marge de 2 pixels.
		couleur.setLayout(new BorderLayout());
		this.add(couleur, BorderLayout.CENTER);
		
		// un sous label pour placer un texte.
		labelTexte =  new JLabel("", JLabel.CENTER);
		labelTexte.setPreferredSize(WindowSwing.DIMCEL3);
		couleur.add(labelTexte, BorderLayout.CENTER);
		
		setTexte("");
	}
	

	/**
	 * Transforme la case en case noire.
	 */
	public void setNoire() {
		// la case a un contour
		this.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
		// pas de texte
		this.labelTexte.setText("");
		// la zone interne noire
		this.couleur.setBackground(Color.BLACK);
	}

	/**
	 * Transforme la case en case blanche.
	 */
	public void setBlanche() {
		// la case a un contour
		this.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
		// pas de texte
		this.labelTexte.setText("");
		// la zone interne blanche
		this.couleur.setBackground(Color.WHITE);

	}
	
	/**
	 * Transforme la case en case blanche barrée.
	 */
	public void setBarree() {
		// la case a un coutour
		this.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
		// la zone interne est blanche
		this.couleur.setBackground(Color.WHITE);
		// avec un gros X pour barrer la case
		Font labelFont = labelTexte.getFont();
		labelTexte.setFont(new Font(labelFont.getName(), Font.PLAIN, FONTZIZEX));
		labelTexte.setText("X");
		
	}
	
	
	/**
	 * Transforme la case en case avec un texte.
	 * Utilisée pour le message d'information sur la bande du haut, mais aussi
	 * pour les indices.
	 * @param texte
	 *        le texte ) écrire dans la case. 
	 */
	public void setTexte(final String texte) {
		// la case n'a pas de contour
		this.setBorder(BorderFactory.createEmptyBorder());
		// la zone interne est blanche
		this.couleur.setBackground(Color.WHITE);
		// elle contient un texte.
		Font labelFont = labelTexte.getFont();
		labelTexte.setFont(new Font(labelFont.getName(), Font.PLAIN, FONTZIZE));
		labelTexte.setText(texte);
		
	}

}

