package tp4509.graphique;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JPanel;

public class ImageSolution extends JPanel{
	/**
	 * Taille d'un pixel de l'image Picross (un pixel de l'image occupe PIXELSIZE pixels sur l'écran).
	 */
	private static final int PIXELSIZE = 5;

	/**
	 * largeur (en nombre de cases) de la grille de jeu.
	 */
	private int largeur;
	/**
	 * hauteur (en nombre de cases) de la grille de jeu.
	 */
	private int hauteur;

	/**
	 * valeurs des pixels de l'image solution.
	 */
	private MarqueLocale[][] grilleLocale;

	/**
	 * Contructeur qui initialise les attributs.
	 * @param largeur
	 *        largeur (en nombre de cases) de la grille de jeu.
	 * @param hauteur
	 *        hauteur (en nombre de cases) de la grille de jeu.
	 * @param grilleLocale
	 *        valeurs des pixels de l'image solution.
	 */
	public ImageSolution(final int largeur, final int hauteur, final MarqueLocale[][] grilleLocale) {
		this.largeur = largeur;
		this.hauteur = hauteur;
		this.grilleLocale = grilleLocale;
		int largeurImage = largeur * PIXELSIZE;
		int hauteurImage = hauteur * PIXELSIZE;
		this.setPreferredSize(new Dimension(largeurImage, hauteurImage));
	}


	// méthode appelée par Swing lorsqu'il doit afficher ce Panel.
	@Override
	public void paintComponent(final Graphics g) {
		int posX;
		int posY;
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		for (int col = 0; col < largeur; col++) {
			posX = col * PIXELSIZE;
			for (int lig = 0; lig < hauteur; lig++) {
				posY = lig * PIXELSIZE;
				if (grilleLocale[col][lig] == MarqueLocale.NOIR) {
					g2.setColor(Color.BLACK);
				} else {
					g2.setColor(Color.WHITE);
				}
				g2.fillRect(posX, posY, PIXELSIZE, PIXELSIZE);
			}
		}
	}



}
