//CHECKSTYLE:OFF
package tp4509.graphique;


import static dm4509.common.Log.COMM;
import static dm4509.common.Log.LOG_ON;

import java.awt.Color;
import java.awt.Dimension;

import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.log4j.Level;

import tp4509.picross.PicrossApi;
import tp4509.picross.PicrossColor;
import tp4509.tcp.ProtocoleException;


/**
 * Classe qui gére la fenêtre de l'interface graphique.
 * @author Eric Lallet.
 *
 */
public class WindowSwing extends JPanel implements MouseListener,  PropertyChangeListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;


	/**
	 * dimmenssion (en pixel) des cases de la grille.
	 */
	public static final int TAILLECEL = 30;

	/**
	 * marge (en pixel) pour centrer le texte des indices.
	 */
	public static final int MARGETEXTE = 10;
	/**
	 * marge (en pixel) pour le contour des cases noires de la grille.
	 */
	public static final int MARGECEL = 3;

	/**
	 * taille de la police de caractères pour les indices.
	 */
	public static final int FONTSIZE = 12;

	/**
	 * taille d'une case de la grille.
	 */
	public static final Dimension DIMCEL1 = new Dimension(TAILLECEL, TAILLECEL);
	/**
	 * taille d'une case de la grille moins de contour.
	 */
	public static final Dimension DIMCEL2 = new Dimension(TAILLECEL -2, TAILLECEL - 2);
	/**
	 * taille de la partie coloriée de la case.
	 */
	public static final Dimension DIMCEL3 = new Dimension(TAILLECEL - 6 , TAILLECEL - 6);

	/**
	 * taille totale (en pixel) de la fenêtre SWING.
	 */
	private int hauteurImage, largeurImage;

	/**
	 * largeur (en nombre de cases) de la zone d'indices des lignes.
	 */
	private int largeurIndices;
	/**
	 * hauteur (en nombre de cases) de la zone d'indices des colonnes. 
	 */
	private int hauteurIndices;
	/**
	 * largeur (en nombre de cases) de la grille de jeu.
	 */
	private int largeur;
	/**
	 * hauteur (en nombre de cases) de la grille de jeu.
	 */
	private int hauteur;

	/**
	 * largeur (en nombre de cases) des indices + grille de jeu. 
	 */
	private int largeurTotale;
	/**
	 * hauteur (en nombre de cases) des indices + grille de jeu.
	 */
	private int hauteurTotale;

	/**
	 * grille locale pour stoker les aides locales (cases barées par clic droit).
	 */
	private MarqueLocale[][] grilleLocale;


	/**
	 * référence sur la classe présentant les informations du pricross.
	 */
	private PicrossApi picrossApi;

	/**
	 * case qui fait la bande en haut de la fenètre (zone d'affichage des messages).
	 */
	private CaseGrille messageInfo;
	
	/**
	 * l'affichage des indicess + la grille de jeu.
	 */
	private CaseGrille[][] caseGrille;
	/**
	 * Constructeur: crée la fenètre, place les indices et une grille vide. 
	 * @param picrossApi
	 *            référence sur la classe présentant les informations du pricross.   
	 * @throws IOException
	 *            toutes les exceptions d'entrée/sortie.
	 * @throws ProtocoleException
	 *            exception si un message reçu n'est pas celui attendu.
	 */
	
	public WindowSwing(final PicrossApi picrossApi) throws IOException, ProtocoleException {
		this.picrossApi = picrossApi;

		// récupération des informations sur le picross. Ces méthodes sont bloquantes
		// dans le cas de la version en réseau, tant que les données n'ont pas été reçues 
		// le client.
		largeurIndices = picrossApi.getLargeurIndices();
		hauteurIndices = picrossApi.getHauteurIndices();
		largeur = picrossApi.getLargeur();
		hauteur = picrossApi.getHauteur();
		grilleLocale = new MarqueLocale[largeur][hauteur];

		// on débute avec une grille vide. Même si d'autres joueurs
		// ont déjà noircie des cases, on n'attends pas la reception de
		// toutes les données pour continuer. Elles seront affichées 
		// plus tard dès leur réception.
		for (int col = 0; col < largeur; col++) {
			for (int lig = 0; lig < hauteur; lig++) {
				grilleLocale[col][lig] = MarqueLocale.BLANC;
			}
		}


		largeurTotale = largeur + largeurIndices;
		hauteurTotale = hauteur + hauteurIndices;
		largeurImage = (largeurTotale + 2) * TAILLECEL;
		hauteurImage = (hauteurTotale + 2) * TAILLECEL;
	

		
	//	setPreferredSize(new Dimension(largeurImage, hauteurImage));
		 buildContentPane();

		// mise en place du CallBack de la souris.
		this.addMouseListener(this);
		
		// mise en place du callback qui surveille les modifications des données de l'API
		this.picrossApi.addPropertyChangeListener(this);
		
	}


	/**
	 * dessine la fenêtre de jeu.
	 */
	private void buildContentPane() {
		
		this.setBackground(Color.WHITE);
		
		// gestion des positionnements à la main (avec setBounds).
		this.setLayout(null);
		
		// la bande du haut réserver aux messages d'info.
		messageInfo = new CaseGrille();
		messageInfo.setBounds(0, 0, largeurImage, TAILLECEL);
		this.add(messageInfo);
		
		
		// un marge en bas
		CaseGrille bas = new CaseGrille();
		bas.setBounds(0, hauteurImage - TAILLECEL, largeurImage, TAILLECEL);
		this.add(bas);
		
		// une marge à gauche
		CaseGrille gauche = new CaseGrille();
		gauche.setBounds(0, TAILLECEL, TAILLECEL, hauteurImage - 2 * TAILLECEL);
		this.add(gauche);
		
		// une marge à droite
		CaseGrille droit = new CaseGrille();
		droit.setBounds(largeurImage-TAILLECEL, TAILLECEL, TAILLECEL, hauteurImage - 2 * TAILLECEL);
		this.add(droit);
		
		// la grille (indices + grille de jeu) au milieu
		 caseGrille = new CaseGrille[largeurTotale][hauteurTotale];
		for (int col = 0; col < largeurTotale; col++) {
			for (int lig = 0; lig < hauteurTotale; lig++) {
				caseGrille[col][lig] = new CaseGrille();
				caseGrille[col][lig].setBounds((col + 1) * TAILLECEL, (lig + 1) * TAILLECEL, TAILLECEL, TAILLECEL);
			
				this.add(caseGrille[col][lig]);
			}
		}
		
	}
	
	/**
	 * Déssine l'image de la solution dans une minifenètre.
	 */
	private void dessinerImage() {
		System.out.println("Dessiner image");
		JFrame imageFen = new JFrame("Image");
		JPanel image  = new ImageSolution(largeur, hauteur, grilleLocale);
		imageFen.add(image);
		imageFen.pack();
		imageFen.setVisible(true);
		//image.setResizable(false);
		imageFen.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
	}
	

	// méthode appelée par Swing lorsqu'il doit afficher ce Panel.
	@Override
	public void paintComponent(final Graphics g) {
		super.paintComponent(g);
		
		// affichage des indices sur les colonnes.
		for (int col = 0; col < largeur; col++) {
			for (int lig = 0; lig < hauteurIndices; lig++) {
				int val = picrossApi.getIndicesColonne(col, lig);
				if (val != PicrossApi.SANSINDICE) {
					ecrireIndiceColonne(col, lig, val);
				}
			}
		}

		// affichage des indices sur les lignes.
		for (int col = 0; col < largeurIndices; col++) {
			for (int lig = 0; lig < hauteur; lig++) {
				int val = picrossApi.getIndicesLigne(col, lig);
				if (val != PicrossApi.SANSINDICE) {
					ecrireIndiceLigne(col, lig, val);
				}
			}
		}

		// affichage de la grille de jeu, et modification de la grille locale en fonction 
		// des modifications des autres joeurs.
		for (int col = 0; col < largeur; col++) {
			for (int lig = 0; lig < hauteur; lig++) {
				if (picrossApi.getValeur(col, lig) == PicrossColor.BLACK.ordinal()) {
					grilleLocale[col][lig] = MarqueLocale.NOIR;
				} else {
					// si la case de la grille du pricoss est blanche, on ne change
					// la grille locale que si celle si est noir. 
					// si la grille locale est barrée, et la case blancje,  on la laisse la grille locale barrée.
					if (grilleLocale[col][lig] == MarqueLocale.NOIR) {
						grilleLocale[col][lig] = MarqueLocale.BLANC;
					}
				}
				remplirCase(col, lig, grilleLocale[col][lig]);
			}
		}

		if (picrossApi.estGagne()) {
			dessinerImage();
			this.messageInfo.setTexte("Partie gagnée.");
		} else { 
			if (picrossApi.connexionPerdue()) {
				this.messageInfo.setTexte("Connexion perdue.");
			}
		}
		
	}



	/**
	 * Affiche le texte d'un indice sur les colonnes.
	 * @param col
	 *         colonne dans la grille des indices sur les colonne.
	 * @param lig
	 *         ligne dans la grille des indices sur les colonne.
	 * @param val
	 *         valeur de l'indice.
	 */
	private void ecrireIndiceColonne(final int col, final int lig, final int val) {
		int celLig =  lig;
		int celCol = largeurIndices + col;
		String texte = String.format("%2d", val);
		caseGrille[celCol][celLig].setTexte(texte);
	}


	/**
	 * Affiche le texte d'un indice sur les lignes.
	 * @param col
	 *         colonne dans la grille des indices sur les lignes.
	 * @param lig
	 *         ligne dans la grille des indices sur les lignes.
	 * @param val
	 *         valeur de l'indice.
	 */
	private void ecrireIndiceLigne(final int col, final int lig, final int val) {
		int celLig = hauteurIndices + lig;
		int celCol = col;
		String texte = String.format("%2d", val);
		caseGrille[celCol][celLig].setTexte(texte);
	}

	// CHECKSTYLE:OFF
	// CHECKSTYLE réclame un default dans un switch où il n'en faut pas.
	/**
	 * Affiche une case de la grile de jeu.
	 * @param col
	 *          colonne dans la grille de jeu.
	 * @param lig
	 *          ligne dans la grille de jeu.
	 * @param marque
	 *          valeur de la marque.
	 */
	private void remplirCase(final int col, final int lig, final MarqueLocale marque) {
		int celX = (col  + largeurIndices);
		int celY = (lig + hauteurIndices);

		switch (marque) {
		case BLANC:
			caseGrille[celX][celY].setBlanche();
			break;
		case BLANCMARQUE:
			caseGrille[celX][celY].setBarree();
			break;
		case NOIR:
			caseGrille[celX][celY].setNoire();
			break;
		}


	}
	// CHECKSTYLE:ON



	// redéfinition de la méthode pour fixer la taille de fenêtre de l'application à la taille de ce JPanel.
	@Override
	public Dimension getPreferredSize() {
		return new Dimension(new Dimension(largeurImage, hauteurImage));
	}

	


	// les méthodes de MousseListener
	@Override
	public void mouseClicked(final MouseEvent event) {
		// rien à faire -> action fait par moussePressed
		
	}




	@Override
	public void mousePressed(final MouseEvent event) {

		if (!picrossApi.estGagne() && !picrossApi.connexionPerdue()) {
			// la partie n'est pas fini et la connexion est ok.

			
			int posx = event.getX();
			int posy = event.getY();
			int celx = (posx - TAILLECEL) / TAILLECEL;
			int cely = (posy - TAILLECEL) / TAILLECEL;

			int col =  celx - largeurIndices;
			int lig =  cely - hauteurIndices;
			if (col < 0 || col >= largeur || lig < 0 || lig >= hauteur) {
				// clic en dehors de la grille: rien à faire.
				return;
			}


			switch (event.getButton()) {
			case MouseEvent.BUTTON1:
				// clic gauche: on change la couleur.
				if (grilleLocale[col][lig] == MarqueLocale.NOIR) {
					grilleLocale[col][lig] = MarqueLocale.BLANC;
				} else {
					grilleLocale[col][lig] = MarqueLocale.NOIR;
				}
				picrossApi.fixerCase(col, lig, grilleLocale[col][lig] == MarqueLocale.NOIR ? PicrossColor.BLACK.ordinal() : PicrossColor.NONE.ordinal());

				break;
			case MouseEvent.BUTTON3:
				// clic droit sur case balnche ou barrée: on interchange barrée <-> blanc.
				if (grilleLocale[col][lig] == MarqueLocale.BLANC) {
					grilleLocale[col][lig] = MarqueLocale.BLANCMARQUE;
					break;
				}
				if (grilleLocale[col][lig] == MarqueLocale.BLANCMARQUE) {
					grilleLocale[col][lig] = MarqueLocale.BLANC;
				}
				break;
			default:

			}
			
			this.repaint();

		}
		

	}


	@Override
	public void mouseReleased(final MouseEvent e) {
		// rien à faire

	}


	@Override
	public void mouseEntered(final MouseEvent e) {
		// rien à faire

	}


	@Override
	public void mouseExited(final MouseEvent e) {
		// rien à faire

	}


	@Override
	public void propertyChange(final PropertyChangeEvent arg0) {
		if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
			COMM.trace("[WindowSwing::propertyChange] changement détecté ");
		}
		this.repaint();
		
	}



}