package tp4509.picross;




import java.io.IOException;



/**
 * Classe qui gère une partie de Picross.
 * @author Eric Lallet.
 *
 */
public class Picross {
	/**
	 * la grille à résoudre.
	 */
	private GrillePicross solution;
	/**
	 * la grille de proposition.
	 */
	private GrillePicross proposition;
	
	/**
	 * Initilisalise en chargeant la grille solution à partir d'un fichier.
	 * @param fileName
	 *        nom du fichier contenant la solution.
	 * @throws IOException
	 *        exceptions dues à la gestion des fichiers.
	 * @throws FileFormatException
	 *        exceptions dues à des erreurs de format dans le fichier de solution.
	 */
	public Picross(final String fileName) throws IOException, FileFormatException {
		solution = new GrillePicross(fileName);
		proposition = new  GrillePicross(solution.getLargeur(), solution.getHauteur());
	}
	
	
	@Override
	public String toString() {
		return "" + solution + proposition;
	}

	/**
	 * Inverse une case de la grille des propositions.
	 * @param col
	 *        colonne de la case.
	 * @param lig
	 *        lignes de la case.
	 * @throws IllegalArgumentException
	 *        exception levée quand la case col x lig est en dehors de la grille.
	 */
	public void changerCase(final int col, final int lig) throws IllegalArgumentException {
		proposition.changerCase(col, lig);
	}

	/**
	 * Fixe une case de la grille des propositions.
	 * @param col
	 *        colonne de la case.
	 * @param lig
	 *        lignes de la case.
	 * @param val
	 *        valeur mise dans la case.
	 * @throws IllegalArgumentException
	 *        exception levée quand la case col x lig est en dehors de la grille.
	 */
	public void fixerCase(final int col, final int lig, final int val) throws IllegalArgumentException {
		proposition.fixerCase(col, lig, val);
	}
	


	/**
	 * getter sur la grille de propositions.
	 * @return
	 *        la grille de propositions.
	 */
	public GrillePicross getProposition() {
		return proposition;
	}

	/**
	 * indique si la partie est gagnée.
	 * @return si la partie est gagnée, false sinon.
	 *        true 
	 */
	public boolean estGagne() {
		// deux grilles sont égales si elles générent les même indices.
		return solution.equals(proposition);
	}
		
	/**
	 * getter sur la largeur de la grille.
	 * @return
	 *        la largeur de la grille.
	 */
	public int getLargeur() {
		return solution.getLargeur();
	}
	
	/**
	 * getter sur la hauteur de la grille.
	 * @return
	 *        la hauteur de la grille.
	 */
	public int getHauteur() {
		return solution.getHauteur();
	}
	
	/**
	 * getter sur la hauteur de la zone d'indices sur les colonnes.
	 * @return
	 *        la hauteur de la zone d'indices sur les colonnes.
	 */
	public int getHauteurIndices() {
		return solution.getHauteurIndices();
	}
	
	/**
	 * getter sur la largeur de la zone d'indices sur les lignes.
	 * @return
	 *        la largeur de la zone d'indices sur les lignes.
	 */
	public int getLargeurIndices() {
		return solution.getLargeurIndices();
	}
	
	/**
	 * getter sur une valeur d'une case de la grille de propositions.
	 * @param col
	 *        la colonne de la case.
	 * @param lig
	 *        la ligen de la case.
	 * @return
	 *       la valeur d'une case.
	 */
	public int getValeur(final int col, final int lig) {
		return proposition.getValeur(col, lig);
	}
	
	/**
	 * getter sur une valeur de la zone des indices sur les colonnes.
	 * @param col
	 *        colonne dans la zone d'indices.
	 * @param lig
	 *        ligne dans la zone d'indices.
	 * @return
	 *        valeur de l'indices.
	 */
	public int getIndiceColonne(final int col, final int lig) {
		return solution.getIndiceColonne(col, lig);
	}

	/**
	 * getter sur une valeur de la zone des indices sur les lignes.
	 * @param col
	 *        colonne dans la zone d'indices.
	 * @param lig
	 *        ligne dans la zone d'indices.
	 * @return
	 *        valeur de l'indices.
	 */
	public int getIndiceLigne(final int col, final int lig) {
		return solution.getIndiceLigne(col, lig);
	}

}
