package tp4509.tcp;

import static dm4509.common.Log.COMM;
import static dm4509.common.Log.LOG_ON;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;

import org.apache.log4j.Level;

/**
 * Classe pour gérer la connexion asynchrone (mode non bloquant) du serveur.
 * @author Eric Lallet.
 *
 */
public class ConnexionAsynchrone implements AutoCloseable {
	
	/**
	 * canal relié à la  connexion.
	 */
	private SocketChannel rwChan;
	
	/**
	 * buffer contenant les données en cours de réception.
	 */
	private ByteBuffer buffer;
	
	/**
	 * indique si le buffer contient un message complet prêt à être délivré.
	 */
	private boolean messageComplet = false;
	
	
	/**
	 * Création du client pour la connexion reliée par le canal rwChan.
	 * @param rwChan
	 *        canal relié à la connexion TCP d'un client.
	 * @throws IOException 
	 *        toutes les exceptions d'entrée/sortie.
	 */
	public ConnexionAsynchrone(final SocketChannel rwChan) throws IOException {
		// TODO Question 3.2.2
	}

	
	
	//                     !!!! ATTENTION !!!!
	// La méthode suivante ne transmet aucune exception (pas de throws dans l'entête).
	// Vous devez garder l'entête de cette méthode ainsi, et attraper les exceptions.
	// Comme l'indique le commentaire javadoc: en cas d'exception la méthode retourne -1.
	/**
	 * Avance la lecture du message, et passe messageComplet à true lorsque le message est entièrement reçu.
	 * @return
	 *             le nombre d'octets placés dans le buffer ou 
	 *             -1 si une exception se produit lors de la lecture ou si la connexion est fermée.    
	 */
	public int avancerLecture()  {
		// TODO Question 3.2.3
		
		return 0;
	}
	
	/**
	 * Délivre le message du protocole si celui-ci a été complètement reçu.
	 * La méthode doit pouvoir être appelée plusieurs fois de suite avec le même message.
	 * @return
	 *     Le message du protocole s'il est complètement reçu, null sinon.
	 */
	public MessageProtocole getMessage() {
		// TODO Question 3.2.4
		
		return null;
	}
	
	
	/**
	 * Efface le message précédement reçu pour préparer la réception d'un nouveau message.
	 * Doit être appelé avant de commencer la réception d'un nouveau message.
	 */
	public void clear() {
		messageComplet = false;
		buffer.clear();
	}
	
	
	/**
	 * Indique si le message en cours de réception est complètement reçu et prêt pour la lecture.
	 * @return
	 *        true si le message est complètement reçu et prêt pour la lecture, false sinon. 
	 */
	public boolean messageComplet() {
		return messageComplet;
	}
	
	
	
	/**
	 * ferme le canal de connexion.
	 * @throws IOException
	 *         toutes les exceptions d'entrées/sorties.
	 */
	public void close() throws IOException {
		rwChan.close();
	}
	
	
	/**
	 * Envoie un message du protocole sur la connexion.
	 * @param message
	 *           le message à envoyer.
	 * @return
	 *           le nombre d'octets envoyés.
	 * @throws IOException
	 *           toutes les exceptions d'entrée/sortie.
	 */
	public int envoyer(final MessageProtocole message) throws IOException {
		if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
			COMM.trace("[ConnexionAsynchrone::envoyer] Envoi du message  = " + message);
		}
		return rwChan.write(message.getBuffer());
	}
	
	
	/**
	 * Getter sur le buffer.
	 * @return
	 *        le buffer contenant les données reçues.
	 */
	public ByteBuffer getBuffer() {
		return buffer;
	}
	
	/**
	 * Getter sur la canal.
	 * @return
	 *        la réference sur le TcpSocket qui gère la connexion.
	 */
	public SocketChannel getRwChan() {
		return rwChan;
	}
	
}
