package tp4509.tcp;

import static dm4509.common.Log.COMM;
import static dm4509.common.Log.LOGGER_NAME_COMM;
import static dm4509.common.Log.LOG_ON;

import java.io.IOException;
import java.nio.channels.ServerSocketChannel;


import org.apache.log4j.Level;

import dm4509.common.Log;
import tp4509.picross.FileFormatException;
import tp4509.picross.PicrossApi;

/**
 * Serveur du jeu picross multijoueur.
 * @author Eric Lallet.
 *
 */
public final class ServeurPicross {
	/**
	 * nombre d'arguments attendus par le programme principal.
	 */
	private static final int NBARGS = 2;
	/**
	 * position du port dans argv[].
	 */
	private static  final int PORTARG = 0;
	/**
	 * position du nom du fichier du picross dans  argv[].
	 */
	private static final int FILENAMEARG = 1;
	
	/**
	 * Durée de le pause (en milliseconde) pour le pause de thread.
	 */
	private static final int PAUSE = 5000; 
	
	/**
	 * 
	 */
	private ServeurPicross() {

	}
	/**
	 * Programme principale du serveur.
	 * @param argv
	 *           argv[0] = port TCP du serveur.
	 * @throws IOException
	 *           toutes les exceptions d'entrées/sorties.
	 * @throws FileFormatException 
	 */
	public static void main(final String[] argv) throws IOException, FileFormatException {
		int port = 0;

		ServerSocketChannel listenChannel;


		if (argv.length != NBARGS) {
			System.out.println("usage: ServeurPicross <port> <nomDeFichier>");
			return;
		}
		Log.configureALogger(LOGGER_NAME_COMM, Level.TRACE);
		port = Integer.parseInt(argv[PORTARG]);

		PicrossApi picrossApi = new PicrossApi(argv[FILENAMEARG]);

		listenChannel = createServerSocket(port);
		serverLoop(listenChannel, picrossApi);

	}

	/**
	 * Ouvre un canal serveur non bloquant sur le port TCP numéro port.
	 * @param port
	 *         numéro du port.
	 * @return
	 *         le ServerSocketChannel en attente sur le port numéro port.
	 * @throws IOException
	 *         toutes les exceptions d'entrées/sorties.
	 */
	public static ServerSocketChannel createServerSocket(final int port) throws IOException {
		// TODO Question 3.4.2

		return null;

	}

	/**
	 * Boucle du serveur qui gère la connexion de nouveaux clients et l'échange avec tous les clients déjà connectés.
	 * @param listenChannel
	 *        le ServerSocketChannel en attente des nouveaux clients.
	 * @param picrossApi
	 *        référence de la classe d'accès au picross à résoudre.
	 * @throws IOException 
	 *         toutes les exceptions d'entrées/sorties.
	 */
	private static void serverLoop(final ServerSocketChannel listenChannel, final PicrossApi picrossApi) throws IOException {
		// TODO Question 3.4.3
	}



	/**
	 * Envoi l'état complet du picross (tailles, indices, grille).
	 * @param client
	 *              la connexion reliée au client qui vient de se connecter.
	 * @param picrossApi
	 *              référence de la classe d'accès au picross à résoudre.
	 * @throws IOException
	 *              toutes les exceptions d'entrée/sortie.
	 */
	private static void envoyerPicross(final ConnexionAsynchrone client, final PicrossApi picrossApi) throws IOException {
		int largeur =  picrossApi.getLargeur();
		int hauteur = picrossApi.getHauteur();
		int largeurIndices =  picrossApi.getLargeurIndices();
		int hauteurIndices =  picrossApi.getHauteurIndices();
		MessageProtocole message;

		// envoi des tailles
		if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
			COMM.trace("[ServeurPicross::envoyerPicross] Envoi de la taille de la grille:" + largeur + " X " + hauteur + ".");
		}
		message = new MessageProtocole(Protocole.TAILLE_GRILLE, largeur, hauteur, 0);
		client.envoyer(message);

		if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
			COMM.trace("[ServeurPicross::envoyerPicross] Envoi de la taille des indices:" + largeurIndices + " X " + hauteurIndices + ".");
		}
		message = new MessageProtocole(Protocole.TAILLE_INDICES, largeurIndices, hauteurIndices, 0);
		client.envoyer(message);



		// envoi des indices sur les colonnes
		if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
			COMM.trace("[ServeurPicross::envoyerPicross] Envoie des incides sur les colonnes.");
		}
		for (int col = 0;  col < largeur; col++) {
			for (int lig = 0; lig < hauteurIndices; lig++) {
				int val = picrossApi.getIndicesColonne(col, lig);
				if (val > 0) {
					message = new MessageProtocole(Protocole.INDICE_COLONNE, col, lig, val);
					client.envoyer(message);
				}
			}
		}


		// envoi des indices sur les lignes
		if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
			COMM.trace("[ServeurPicross::envoyerPicross] Envoie des incides sur les lignes.");
		}
		for (int col = 0; col < largeurIndices; col++) {
			for (int lig = 0;  lig < hauteur; lig++) {
				int val = picrossApi.getIndicesLigne(col, lig);
				if (val > 0) {
					message = new MessageProtocole(Protocole.INDICE_LIGNE, col, lig, val);
					client.envoyer(message);
				}
			}
		}


		// envoi de la grille de propositions
		if (LOG_ON && COMM.isEnabledFor(Level.TRACE)) {
			COMM.trace("[ServeurPicross::envoyerPicross] Envoie de la grille.");
		}
		for (int lig = 0; lig < hauteur; lig++) {
			for (int col = 0; col < largeur; col++) {
				message = new MessageProtocole(Protocole.VALEUR_GRILLE_SERVEUR, col, lig, picrossApi.getValeur(col, lig));
				client.envoyer(message);
			}
		}


	}

}
